'' =================================================================================================
''
''   File....... jm_nunchuk_servos.spin
''   Purpose.... Runs two servos (setup for C3 board)
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2011 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.term
''   Started.... 
''   Updated.... 06 JAN 2011
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1  = 31
  TX1  = 30
  SDA  = 29
  SCL  = 28

  SVO4 = 7                                                      ' Tri-Port of C3  
  SVO3 = 6
  SVO2 = 5
  SVO1 = 4                                                     


con

  SVO_MIN = 1_000
  SVO_MAX = 2_000


con 

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control 
  #14, GOTOX, GOTOY, CLS      


obj

  wii  : "jm_nunchuk_ez"                                       ' polled Wii nunchuk
  term : "fullduplexserial"                                     ' for terminal output


pub main | t, xidle, yidle, pos

  wii.init(SCL, SDA)                                            ' setup Wii 
  term.start(RX1, TX1, %0000, 115_200)                          ' start terminal cog
  pause(1)

  term.str(string(CLS, "Nunchuk Demo", CR, "-- X/Y Servos"))

  wii.scan                                                      ' read idle positions
  xidle := wii.joyx
  yidle := wii.joyy                                                

  t := cnt
  repeat
    wii.scan                                                    ' scan Wii nunchuk
    term.str(string(HOME, LF, LF, LF))                          ' position for report 
    
    term.str(string("Joy X... "))
    pos := calibrate(wii.joyx, xidle, 255)                      ' get/correct input
    term.dec(pos)                                               ' display input
    term.str(string("  "))                                      ' clean-up last output
    term.tx(GOTOX)                                              ' move to col 15
    term.tx(15)
    pos := scale(pos, SVO_MIN, SVO_MAX)                         ' move servo with joystick
    term.dec(pos)                                               ' display servo timing                                               
    term.tx(CLREOL)                                             ' cleanup line 
    term.tx(CR)                                                  
                                                                 
    pulsout(SVO1, pos)                                          ' refresh servo 

    term.str(string("Joy Y... "))
    pos := calibrate(wii.joyy, yidle, 255)
    term.dec(pos)
    term.str(string("  "))
    term.tx(GOTOX)
    term.tx(15)
    pos := scale(pos, SVO_MIN, SVO_MAX)
    term.dec(pos)
    term.tx(CLREOL)
    term.tx(CR)

    pulsout(SVO2, pos)
              
    
    waitcnt(t += constant(20 * MS_001))                         ' use 20ms for servos


pub calibrate(in, idle, range) | mid

'' Calibrate joystick input based on starting position
'' --    in: uncalibrated input
'' --  idle: startup (idle) reading
'' -- range: max value for joystick

  mid := range >> 1                                             ' 1/2 point

  if (in == idle)
    result := in                                                ' at idle point
  elseif (in < idle)                                                                           
    result := mid * in / idle                                   ' below idle point
  else                                                                 
    result := mid * (in-idle) / (range-idle) + mid              ' above idle point


pub scale(raw, minOut, maxOut)

'' Scales raw (0 to 255) value to new range: minOut to maxOut

  raw := 0 #> raw <# 255                                        ' constrain input

  if (minOut < maxOut)                                          ' if good range
    return ((raw * (maxOut - minOut)) / 255) + minOut
  else
    return raw
    

pub pulsout(pin, width)

'' Creates high-going pulse on pin
'' -- pulse width specified in microseconds

  phsa := 0                                                     ' setup ctra
  frqa := 1                                                      

  dira[pin] := 1                                                ' make pin an output

  ctra := (%00100 << 26) | pin                                  ' use PWM/NCO mode
  phsa := -(width * US_001)                                     ' set pulse timing       
  repeat while ina[pin]                                         ' let it finish       
  ctra := 0                                                     ' disable cnta
 
 
pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)

      
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}